/* ORS_AttenGlass.sl version 2 by Owen Roberts 2005 July 09
   Surface shader for Sitex Graphics AIR 3.1
   Simulates glass and other transparent dielectrics with exponential attenuation of transmission.
   2005 July 05: original version.
   2005 July 09: version 2 - added parameters for ambient and diffuse.
*/

#define BLACK color( 0 )
#define WHITE color( 1 )

color OR_ExpAttenuate( color linearAtten; float dist; ) {
   float r = comp( linearAtten, 0 );
   r = (r == 0) ? 1 : exp( -dist * r );
   float g = comp( linearAtten, 1 );
   g = (g == 0) ? 1 : exp( -dist * g );
   float b = comp( linearAtten, 2 );
   b = (b == 0) ? 1 : exp( -dist * b );
   return color( r, g, b );
}

surface ORS_AttenGlass(
   float ambientIntensity = 0, diffuseIntensity = 0;
   float specIntensity = 1, specRoughness = 0.05, specSharpness = 0.8;
   float reflectIntensity = 1, refractIntensity = 1;
   float outsideIOR = 1, insideIOR = 1.5;
   color outsideTransmitColor = WHITE;
   color insideTransmitColor = color( 0.94, 0.97, 0.96 );
   float transmitDistanceScale = 1;
   float reflectBlur = 0, reflectSamples = 1;
   float refractBlur = 0, refractSamples = 1;
   float maxTraceDistance = 1e9;
   float maxReflectBounces = 31;
   float shadowPower = 1;
   output varying color __ambient = BLACK;
   output varying color __diffuse = BLACK;
   output varying color __specular = BLACK;
   output varying color __reflect = BLACK;
   output varying color __refract = BLACK;
)
{
   normal Nf = faceforward( normalize( N ), I );
   vector In = normalize( I );
   vector V = -In;
   float isInside = (I.N > 0) ? 1 : 0;
   
   color attenColor = (isInside == 1) ? insideTransmitColor : outsideTransmitColor;
   if( attenColor != WHITE ) {
      attenColor = OR_ExpAttenuate( (WHITE - attenColor), (transmitDistanceScale * length( I )) );
   }

   if( isshadowray() == 1 ) {
      if( isInside == 1 ) {
         if( shadowPower == 1 ) {
            Oi = WHITE - (refractIntensity * V.Nf * attenColor);
         } else {
            Oi = WHITE - (pow( (refractIntensity * V.Nf), shadowPower ) * attenColor);
         }
      } else {
         Oi = BLACK;
      }
      Ci = BLACK;
   } else {
      __ambient = ambientIntensity * Cs * attenColor * ambient();
      __diffuse = diffuseIntensity * Cs * attenColor * diffuse( Nf );
      //__specular = specIntensity * attenColor * sharpspecular( Nf, V, specRoughness, specSharpness );
      __specular = specIntensity * attenColor * specular( Nf, V, specRoughness );
      float kr = (raylevel() < maxReflectBounces) ? reflectIntensity : 0;
      float kt = refractIntensity;
      float eta, fkr, fkt;
      vector R, T;
      color weight;
      eta = (isInside == 1) ? (insideIOR / outsideIOR) : (outsideIOR / insideIOR);
      fresnel( In, Nf, eta, fkr, fkt, R, T );
      kr *= fkr;
      kt *= (1 - fkr);
      if( kr > 0 ) {
         weight = kr * attenColor;
         __reflect = weight * environment( "raytrace", R, "blur", reflectBlur, "samples", reflectSamples, "maxdist", maxTraceDistance );
      }
      if( kt > 0 ) {
         weight = kt * attenColor;
         __refract = weight * environment( "raytrace", T, "blur", refractBlur, "samples", refractSamples, "maxdist", maxTraceDistance );
      }
      Oi = WHITE;
      Ci = __ambient + __diffuse + __specular + __reflect + __refract;
   }
} 